'' =================================================================================================
''
''   File....... pgm_reload.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 
''
'' =================================================================================================


CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000


CON

  SD_DO    = 0
  SD_CLK   = 1
  SD_DI    = 2
  SD_CS    = 3

  PG_SIZE  = 64

  DEMO     = 1   


CON

  #1, HOME, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR, #16, CLS      ' PST formmatting control


OBJ

  term : "fullduplexserial"
  i2c  : "jm_i2c_driven" 
  sd   : "fsrw"


VAR

  byte  buf[PG_SIZE]
  

DAT

NewFirmware             byte "update.pgm", 0    


PUB main | idx, pattern

  term.start(31, 30, %0000, 115_200)                            ' start a terminal
  pause(1)
  term.tx(CLS)

  i2c.init(i2c#BOOTSCL, i2c#BOOTSDA)

  checkupdate

  dira[23..16]~~

  repeat
    case DEMO
      1 : repeat idx from 0 to 7
            outa[23..16] := Sequence1[idx]
            pause(125)

      2 : repeat idx from 0 to 3
            outa[23..16] := Sequence2[idx]
            pause(250)      


PUB checkupdate | check, fsize, addr

'' Checks for mounted SD card and presence of "update.pgm"
'' -- if "update.pgm" is on SD card the contents are loaded into the EEPROM
''    and the Propeller is rebooted

  check := \sd.mount(0)
  if (check == 0)
    term.str(string("1. SD card mounted.", CR))
    check := sd.popen(@NewFirmware, "r")                        ' look for update
    if (check == 0)
      term.str(string("2. Found update UPDATE.PGM.", CR))
      fsize := sd.get_filesize                                  
      if (fsize == 32768)                                       ' check 32K (ee image)
        term.str(string("   -- size is correct", CR))
        term.str(string("3. Updating EEPROM...", CR))
        addr~ 
        repeat constant(32768 / PG_SIZE)                        ' loop through pages
          sd.pread(@buf, PG_SIZE)                               ' read one
          term.tx(14)                                           ' report targe address
          term.tx(0)
          term.str(string("   -- $"))
          term.hex(addr, 4)
          check := i2c.putpagex($A0, addr, PG_SIZE, @buf)       ' write it
          if (check == i2c#NAK)                                 ' if problem
            term.str(string("   -- EE write error", CR))
            quit                                                ' abort
          addr += PG_SIZE                                       ' else next page
        term.str(string(CR, "   -- update complete", CR))
        pause(2_000)
        sd.popen(@NewFirmware, "d")                             ' delete update file
        sd.pclose
        reboot                                                  ' restart device
      else
        term.str(string("   -- bad file size; update aborted", CR))
    else
      term.str(string("   -- no update file; update aborted", CR))
  else
    term.str(string("No SD card; update aborted.", CR))   
      
  sd.pclose
    

PUB pause(ms) | t

  t := cnt                                                      ' sync with system counter
  repeat ms
    waitcnt(t += MS_001)


DAT

Sequence1               byte    %0000_0001, %0000_0010, %0000_0100, %0000_1000
                        byte    %0001_0000, %0010_0000, %0100_0000, %0000_0000

Sequence2               byte    %0001_1000, %0010_0100, %0100_0010, %1000_0001                        

                       
DAT

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  